/* 
          LTR210.
            ,    
           :
    ltr210_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (  )
     :    ,       

              ,
        2  ,     
     Frame< >.txt (    )   .

         ,   
     .

           Windows   CTRL+C  Linux

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr210api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .
*/


#include "ltr/include/ltr210api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif



/*       */
#define RECV_TOUT       10000
/*           ,     */
#define KEEPALIVE_TOUT  10000

/*  ,       */
#define FRAME_SAVE_TO_FILE

/*    .   =>    */
#define LTR210_FPGA_FIRM_FILE ""

/*      */
static int f_out = 0;


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;

    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}

/*    */
static void APIENTRY f_firm_upd_cb(void* cb_data, TLTR210* hnd, DWORD done_size, DWORD full_size) {
    printf("."); fflush(stdout);
}


static int f_save_to_file(int frame_num, double* data, int size, int ch_cnt) {
    char fname[40];
    FILE* f;
    int i,ch;
    int err = 0;

    /*      .     */
    sprintf(fname, "Frame%d.txt", frame_num);
    f = fopen(fname, "w");
    if (f != NULL) {
        for (i = 0; (i < size) && !err; i += ch_cnt) {
            for (ch=0; (ch < ch_cnt) && !err; ch++) {
                if (fprintf(f, "%.5f ", data[i+ch]) <= 0)
                    err = 1;
            }

            if (fprintf(f, "\n")<=0)
                err = 1;
        }
        fclose(f);

        if (err) {
            fprintf(stderr, "   !\n");
        }
    } else {
        printf("  !\n");
    }
    return err;
}


#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


int main(int argc, char **argv) {
    INT err = LTR_OK;
    TLTR210 hltr210;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err == LTR_OK) {
        LTR210_Init(&hltr210);
        /*     */
        err = LTR210_Open(&hltr210, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR210_GetErrorString(err));
        } else {
            /*     ,     */
            printf("  !\n");
            printf("       = %s\n", hltr210.ModuleInfo.Name);
            printf("        = %s\n", hltr210.ModuleInfo.Serial);
            printf("   PLD         = %d\n", hltr210.ModuleInfo.VerPLD);            

            /* ,   .   -  */
            if (LTR210_FPGAIsLoaded(&hltr210) != LTR_OK) {
                printf("   \n");
                err = LTR210_LoadFPGA(&hltr210, LTR210_FPGA_FIRM_FILE, f_firm_upd_cb, NULL);
                if (err != LTR_OK) {
                    fprintf(stderr, "    .  %d (%s)\n",
                            err, LTR210_GetErrorString(err));
                } else {
                    printf("\n   .\n");
                }
            }

            if (err == LTR_OK) {
                printf("   = %d\n", hltr210.ModuleInfo.VerFPGA);
            }

            fflush(stdout);

            if (err == LTR_OK) {
                DWORD *wrds = NULL;
                double *data = NULL;

                /* -------------    --------------- */

                /*    */
                hltr210.Cfg.Ch[0].Enabled = 1;
                hltr210.Cfg.Ch[0].Range = LTR210_ADC_RANGE_10;
                hltr210.Cfg.Ch[0].Mode = LTR210_CH_MODE_ACDC;
                hltr210.Cfg.Ch[1].Enabled = 1;
                hltr210.Cfg.Ch[1].Range = LTR210_ADC_RANGE_0_5;
                hltr210.Cfg.Ch[1].Mode = LTR210_CH_MODE_AC;

                hltr210.Cfg.SyncMode = LTR210_SYNC_MODE_PERIODIC;

                /*   */
                hltr210.Cfg.FrameSize = 10000;
                /*   */
                hltr210.Cfg.HistSize = 100;


                /*      
                     */
                hltr210.Cfg.Flags = LTR210_CFG_FLAGS_KEEPALIVE_EN |
                        LTR210_CFG_FLAGS_WRITE_AUTO_SUSP;


                /*     - 10  */
                err = LTR210_FillAdcFreq(&hltr210.Cfg, 10000000., 0, NULL);
                /*    1  (  ) */
                if (err == LTR_OK)
                    err = LTR210_FillFrameFreq(&hltr210.Cfg, 1., NULL);
                /*     */
                if (err == LTR_OK) {
                    err = LTR210_SetADC(&hltr210);
                    if (err != LTR_OK) {
                        fprintf(stderr, "    :  %d (%s)\n",
                                err, LTR210_GetErrorString(err));
                    }
                }


                if (err == LTR_OK) {
                    /*       */
                    err = LTR210_MeasAdcZeroOffset(&hltr210, 0);
                    if (err != LTR_OK) {
                        fprintf(stderr, "     :  %d (%s)\n",
                                err, LTR210_GetErrorString(err));
                    }
                }


                if (err == LTR_OK) {
                    /*        */
                    wrds = (DWORD*)malloc(hltr210.State.RecvFrameSize*sizeof(wrds[0]));
                    data = (double*)malloc(hltr210.State.RecvFrameSize*sizeof(data[0]));

                    if ((wrds == NULL) || (data == NULL)) {
                        fprintf(stderr, "  !\n");
                        err = LTR_ERROR_MEMORY_ALLOC;
                    }
                }

                if (err == LTR_OK) {
                    err = LTR210_Start(&hltr210);
                    if (err != LTR_OK) {
                        fprintf(stderr, "    !  %d (%s)\n",
                                err, LTR210_GetErrorString(err));
                    } else {
                        printf("  .    %s\n",
#ifdef _WIN32
                               " "
#else
                               "CTRL+C"
#endif
                               );
                        fflush(stdout);
                    }
                }

                while (!f_out && (err == LTR_OK)) {
                    DWORD evt;
                    INT recv_cnt=0;
                    static int frame_num=0;

                    /*      */
                    if (err == LTR_OK)
                        err = LTR210_WaitEvent(&hltr210, &evt, NULL, 100);
                    if (err == LTR_OK) {
                        /* ,     */
                        switch (evt) {
                            case LTR210_RECV_EVENT_SOF:
                                /*   .   
                                 *    Recv,    
                                 *      */
                                recv_cnt = LTR210_Recv(&hltr210, wrds, NULL,
                                                       hltr210.State.RecvFrameSize, RECV_TOUT);
                                /*       */
                                if (recv_cnt < 0) {
                                    err = recv_cnt;
                                    fprintf(stderr, "   :  %d (%s)\n",
                                            err, LTR210_GetErrorString(err));
                                } else if (recv_cnt < (INT)hltr210.State.RecvFrameSize) {
                                    fprintf(stderr, "  ,    !  %d,  %d\n",
                                            hltr210.State.RecvFrameSize, recv_cnt);
                                    err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                                } else {
                                    TLTR210_FRAME_STATUS frame_st;
                                    /*          */
                                    err = LTR210_ProcessData(&hltr210, wrds, data, &recv_cnt,
                                                             LTR210_PROC_FLAG_VOLT |
                                                             LTR210_PROC_FLAG_AFC_COR |
                                                             LTR210_PROC_FLAG_ZERO_OFFS_COR,
                                                             &frame_st, NULL);
                                    if (err != LTR_OK) {
                                        printf("  !  %d (%s)\n",
                                               err, LTR210_GetErrorString(err));
                                    } else {
                                        int i;
                                        frame_num++;
                                        /*      ,
                                         *      */
                                        printf("   %d:  : ", frame_num);
                                        for (i=0; (i < 2) && (i < recv_cnt); i++) {
                                            printf(" %.4f ", data[i]);
                                        }
                                        printf("\n");
                                        fflush(stdout);
#ifdef FRAME_SAVE_TO_FILE
                                        f_save_to_file(frame_num, data, recv_cnt,
                                            (hltr210.Cfg.Ch[0].Enabled && hltr210.Cfg.Ch[1].Enabled) ? 2 : 1);
#endif
                                    }
                                }
                                break;
                            case LTR210_RECV_EVENT_TIMEOUT:
                                if (hltr210.Cfg.Flags & LTR210_CFG_FLAGS_KEEPALIVE_EN) {
                                    /*    ,  ,      
                                     *   */
                                    DWORD interval;
                                    err = LTR210_GetLastWordInterval(&hltr210, &interval);
                                    if (err == LTR_OK) {
                                        if (interval > KEEPALIVE_TOUT) {
                                            fprintf(stderr, "        .   !\n");
                                            err = LTR210_ERR_KEEPALIVE_TOUT_EXCEEDED;
                                        }
                                    } else {
                                        fprintf(stderr, "         .  %d (%s)\n",
                                                err, LTR210_GetErrorString(err));
                                    }
                                }
                            default:
                                break;
                        }
                    } else {
                        fprintf(stderr, "     !  %d (%s)\n",
                                err, LTR210_GetErrorString(err));
                    }

#ifdef _WIN32
                    /*      */
                    if (err == LTR_OK) {
                        if (_kbhit())
                            f_out = 1;
                    }
#endif
                } // while(!f_out && (err == LTR_OK))

                /*    ,     */
                if (hltr210.State.Run) {
                    INT stoperr = LTR210_Stop(&hltr210);
                    if (stoperr == LTR_OK) {
                        printf("  !\n");
                    } else {
                        fprintf(stderr, "   .  %d (%s)\n",
                                err, LTR210_GetErrorString(err));
                        if (err == LTR_OK)
                            err = stoperr;
                    }
                }

                free(wrds);
                free(data);
            }
        }
        LTR210_Close(&hltr210);
    }


    return err;
}
